﻿using GridTest.ViewModel;
using Infragistics.Windows.DataPresenter;
using System;
using System.Collections.Generic;
using System.Timers;
using System.Windows;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Threading;
using System.Globalization;

namespace GridTest
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        DispatcherTimer timer = new DispatcherTimer();
        ApplicationViewModel ApplicationDataViewModel; //Session Data Context

        public List<GridData> matrixRowData;

        public MainWindow()
        {
            InitializeComponent();
            matrixTuningComboBox.SelectedIndex = 0;

            CreateData();
            ApplicationDataViewModel = (ApplicationViewModel)base.DataContext; //Set The DataContext
            ApplicationDataViewModel.PropertyChanged += ApplicationDataViewModel_PropertyChanged;
            testGrid.DataSource = matrixRowData;

            timer.Interval = new TimeSpan(0, 0, 0, 0, 100);
            timer.Tick += Timer_Tick;
            timer.Start();
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            Random rand = new Random();
            ApplicationDataViewModel.ColIndex = rand.Next(0, 15);
            ApplicationDataViewModel.RowIndex = rand.Next(0, 15);
        }

        private void ApplicationDataViewModel_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "ColIndex")
            {
                if (ApplicationDataViewModel.LockColIndex > -1)
                {
                    Style lockStyle = this.Resources["CellLockedStyle"] as Style;
                    Style disabledStyle = this.Resources["CellDisabledStyle"] as Style;

                    foreach (Field f in testGrid.DefaultFieldLayout.Fields)
                    {
                        if (f.CellValuePresenterStyle != disabledStyle)
                        {
                            f.CellValuePresenterStyle = disabledStyle;
                        }
                    }

                    testGrid.DefaultFieldLayout.Fields[ApplicationDataViewModel.LockColIndex].CellValuePresenterStyle = lockStyle;
                }
                else
                {
                    Style greenStyle = this.Resources["CellGreenStyle"] as Style;
                    foreach (Field f in testGrid.DefaultFieldLayout.Fields)
                    {
                        if (f.CellValuePresenterStyle != null)
                        {
                            f.CellValuePresenterStyle = null;
                        }
                    }
                    testGrid.DefaultFieldLayout.Fields[ApplicationDataViewModel.ColIndex].CellValuePresenterStyle = greenStyle;
                }
            }
            else if (e.PropertyName == "TuningMatrixIndex")
            {
                FieldSettings set = testGrid.FieldSettings;
                FieldLayoutSettings lay = testGrid.FieldLayoutSettings;

                if (matrixTuningComboBox.SelectedIndex == 1)
                {
                    set.CellClickAction = CellClickAction.SelectCell;
                    lay.SelectionTypeCell = Infragistics.Windows.Controls.SelectionType.ExtendedAutoDrag;
                }
                else
                {
                    set.CellClickAction = CellClickAction.EnterEditModeIfAllowed;
                    lay.SelectionTypeCell = Infragistics.Windows.Controls.SelectionType.Single;
                }

                testGrid.FieldSettings = set;
                testGrid.FieldLayoutSettings = lay;
            }
        }

        public void CreateData()
        {
            byte[] bytes = new byte[256];

            for (int i = 0; i < 256; i++)
            {
                if (i == 1)
                {
                    bytes[i] = 15;
                }
                else
                {
                    bytes[i] = 0;
                }
            }

            int rowCounter = 16;
            int colCounter = 0;
            int index = 0;

            matrixRowData = new List<GridData>();
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());
            matrixRowData.Add(new GridData());

            foreach (byte b in bytes)
            {
                switch (colCounter)
                {
                    case 0:
                        matrixRowData[rowCounter - 1].Col1 = b;
                        break;
                    case 1:
                        matrixRowData[rowCounter - 1].Col2 = b;
                        break;
                    case 2:
                        matrixRowData[rowCounter - 1].Col3 = b;
                        break;
                    case 3:
                        matrixRowData[rowCounter - 1].Col4 = b;
                        break;
                    case 4:
                        matrixRowData[rowCounter - 1].Col5 = b;
                        break;
                    case 5:
                        matrixRowData[rowCounter - 1].Col6 = b;
                        break;
                    case 6:
                        matrixRowData[rowCounter - 1].Col7 = b;
                        break;
                    case 7:
                        matrixRowData[rowCounter - 1].Col8 = b;
                        break;
                    case 8:
                        matrixRowData[rowCounter - 1].Col9 = b;
                        break;
                    case 9:
                        matrixRowData[rowCounter - 1].Col10 = b;
                        break;
                    case 10:
                        matrixRowData[rowCounter - 1].Col11 = b;
                        break;
                    case 11:
                        matrixRowData[rowCounter - 1].Col12 = b;
                        break;
                    case 12:
                        matrixRowData[rowCounter - 1].Col13 = b;
                        break;
                    case 13:
                        matrixRowData[rowCounter - 1].Col14 = b;
                        break;
                    case 14:
                        matrixRowData[rowCounter - 1].Col15 = b;
                        break;
                    case 15:
                        matrixRowData[rowCounter - 1].Col16 = b;
                        break;
                }
                rowCounter--;

                if (rowCounter < 1)
                {
                    rowCounter = 16;
                    colCounter++;
                }

                index++;
            }
        }

        private void testGrid_PreviewKeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == System.Windows.Input.Key.Space)
            {
                if (ApplicationDataViewModel.LockColIndex > -1)
                {
                    ApplicationDataViewModel.LockColIndex = -1;
                }
                else
                {
                    if (testGrid.ActiveCell == null)
                    {
                        ApplicationDataViewModel.LockColIndex = testGrid.SelectedItems.Fields[0].Index;
                    }
                    else
                    {
                        ApplicationDataViewModel.LockColIndex = testGrid.ActiveCell.Field.Index;
                    }
                }

                e.Handled = true;
            }
            else
            {
                if (ApplicationDataViewModel.LockColIndex > -1)
                {
                    switch (e.Key)
                    {
                        case System.Windows.Input.Key.Up:
                            this.testGrid.ExecuteCommand(DataPresenterCommands.CellAbove);
                            this.testGrid.ExecuteCommand(DataPresenterCommands.StartEditMode);

                            e.Handled = true;
                            break;
                        case System.Windows.Input.Key.Down:
                            this.testGrid.ExecuteCommand(DataPresenterCommands.CellBelow);
                            this.testGrid.ExecuteCommand(DataPresenterCommands.StartEditMode);

                            e.Handled = true;
                            break;
                        case System.Windows.Input.Key.Left:
                            testGrid.ActiveCell.Value = Convert.ToInt32(testGrid.ActiveCell.Value) - 1;

                            e.Handled = true;
                            break;
                        case System.Windows.Input.Key.Right:
                            testGrid.ActiveCell.Value = Convert.ToInt32(testGrid.ActiveCell.Value) + 1;

                            e.Handled = true;
                            break;
                    }
                }
                else
                {
                    if (matrixTuningComboBox.SelectedIndex == 6 || matrixTuningComboBox.SelectedIndex == 7)
                    {
                        switch (e.Key)
                        {
                            case System.Windows.Input.Key.Add:
                            case System.Windows.Input.Key.OemPlus:
                                foreach (var c in testGrid.SelectedItems.Cells)
                                {
                                    c.Value = Convert.ToInt32(c.Value) + 1;
                                }
                                e.Handled = true;
                                break;
                            case System.Windows.Input.Key.Subtract:
                            case System.Windows.Input.Key.OemMinus:
                                foreach (var c in testGrid.SelectedItems.Cells)
                                {
                                    c.Value = Convert.ToInt32(c.Value) - 1;
                                }

                                e.Handled = true;
                                break;
                        }                        
                    }
                    else
                    {
                        switch (e.Key)
                        {
                            case System.Windows.Input.Key.Add:
                            case System.Windows.Input.Key.OemPlus:
                                testGrid.ActiveCell.Value = Convert.ToInt32(testGrid.ActiveCell.Value) + 1;

                                e.Handled = true;
                                break;
                            case System.Windows.Input.Key.Subtract:
                            case System.Windows.Input.Key.OemMinus:
                                testGrid.ActiveCell.Value = Convert.ToInt32(testGrid.ActiveCell.Value) - 1;

                                e.Handled = true;
                                break;
                        }
                    }
                }
            }
        }

        private void matrixTuningComboBox_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            try
            {
                ApplicationDataViewModel.TuningMatrixIndex = matrixTuningComboBox.SelectedIndex;
            }
            catch (Exception ex)
            {
                string error = ex.ToString();
            }
        }
    }

    public class RowStyleConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, CultureInfo culture)
        {
            int rowindex = (int)values[0];
            int lockedColIndex = (int)values[1];
            DataRecord row = values[2] as DataRecord;

            if (row != null)
            {
                if (row.Index == rowindex)
                {
                    if (lockedColIndex == -1)
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
